#include <stdint.h>
#include <uuid/uuid.h>


/* Device flag definitions */
#define INITIALIZED     0x0001          // The MBR is initialized
#define SHARED          0x0002          // The device is marked shared for clustering
#define RAID            0x0004          // This is a RAID device (0,1,or 5)
#define MIRROR          0x0008          // This is a RAID1 device
#define ENABLED         0x0010          // This device is enabled on this node (RAID only)
#define IS_GPT          0x0020          // This has a GPT partition table
#define IS_MSDOS        0x0040          // This has an MSDOS partition table
#define IS_LVM          0x0080          // This device has LVM partitions on it
#define IS_BOOT         0x0100          // This is a boot device
#define HAS_SWAP        0x0200          // This device has a swap partition
#define MULTIPATH       0x0400          // This is a multipath device
#define IS_CSM          0x0800          // This has a CSM stamp
#define IS_LVM_VOLUME   0x1000          // This has an LVM2 stamp at the beginning
#define IS_INVALID      0x8000          // This RAID device has duplicate segments

/* Partition flag definitions */
#define FREESPACE       0x0001          // This is a free partition
#define SHARED          0x0002          // This partition is on a shared disk
#define RAID            0x0004          // This partition is part of a RAID device
#define HAS_POOL        0x0008          // This partition has a pool on it
#define ACTIVE          0x0010          // This partition is active (RAID only)
#define INSYNC          0x0020          // This partition is in sync (RAID only)
#define LOGICAL         0x0040          // This partition is a logical partition
#define IS_LVM          0x0080          // This partition has a PV (LVM)
#define IS_BOOT         0x0100          // This partition is a boot partition
#define EXPANDABLE      0x0200          // This partition can be expanded
#define IS_HOST         0x0400          // This partition hosts NW partitions (similar to extended)
#define IS_MOVE         0x0800          // This partition is move destination
#define IS_DM           0x1000          // Create this partition with a DM object

/* Group flag definitions */
#define IS_LVM          0x0080          // This group is on a logical volume (LVM)
#define IS_MOVE         0x0800          // This group is move destination
#define IS_INVALID      0x8000          // This group has duplicate segments

/* Pool flag definitions */
#define SHARED          0x0002          // This pool is on a shared disk
#define IS_MOUNTED      0x0004          // This pool is mounted
#define IS_SNAPSHOT     0x0008          // This pool is a snapshot
#define HAS_SNAPSHOT    0x0010          // This pool has a snapshot
#define IS_LVM          0x0080          // This pool is on a logical volume (LVM)
#define HAS_MOVE        0x0100          // This pool is being moved
#define IS_INVALID      0x8000          // This pool has an invalid configuration

/* Snapshot flag definitions */
#define SHARED          0x0002          // This snapshot is on a shared disk
#define IS_MOUNTED      0x0004          // This snapshot is mounted
#define IS_SNAPSHOT     0x0008          // This is a snapshot (of course!)
#define IS_VALID        0x0010          // This snapshot is valid in device-mapper
#define WRITEABLE       0x0020          // This snapshot is writeable
#define IS_INVALID      0x8000          // This snapshot has duplicate partitions

/* NLVMVolumeInfo_s flag definitions */
#define SHARED          0x0002          // This volume is on a shared pool
#define IS_MOUNTED      0x0004          // This volume is mounted
#define NO_RENAME       0x0008          // Cannot rename the mount point

/* NLVMLinuxVolumeInfo_s flag definitions */
#define SHARED          0x0002          // This volume is on a shared device
#define IS_MOUNTED      0x0004          // This volume is mounted
#define NO_RENAME       0x0008          // Cannot rename the mount point (NSS only)
#define USE_NAMESPC     0x0010          // Set the name space provided (NSS only)
#define NOAUTO          0x0020          // Set the "noauto" flag
#define VOL_RW          0x0040          // Set the "rw" flag.  If not set, set the "ro" flag
#define IS_LVM          0x0080          // This volume is on a logical volume (LVM)
#define NCP_ENABLE      0x0100          // NCP Enable this volume
#define NCP_MOUNTED     0x0200          // NCP volume is mounted

/* NLVM_DeleteMovePool flag definitions */
#define MOVE_ABORT      0x0001          // Abort the move and return back to original devices

/* Generic mask bits for various functions */
#define DONT_CLEAR_STAMPS  0x40000000   // interal flag only to not restamp
#define SHARE_OVERRIDE     0x10000000   // Use to execute command even without cluster lock
#define NO_NDS             0x08000000   // Do not change NDS object
#define NDSNAME            0x04000000   // The supplied name is an NDS name
#define INVALID_OVERRIDE   0x02000000   // Use to override an invalid pool 
#define FORCE_OVERRIDE     0x01000000   // Use to force a command (if supported) 

/* Device list mask definitions */
#define D_DISK_BIT      0x0001          // Return raw disks (non RAIDs)
#define D_RAID_BIT      0x0002          // Return RAID devices
#define D_NSS_BIT       0x0004          // Return devices that have no LVM
#define D_LVM_BIT       0x0008          // Return devices that are either empty or has LVM
#define D_SHARED_BIT    0x0010          // Return devices that are shared
#define D_NSHARE_BIT    0x0020          // Return devices that are not shared
#define D_MP_BIT        0x0040          // Return multi-path devices (not implemented)
#define ALL_BITS        0xFFFF

/* Partition list mask definitions */
#define DATA_BIT        0x0001          // Return data partitions
#define FREE_BIT        0x0002          // Return free partitions
#define NSS_BIT         0x0004          // Return only NSS type or free that can be made NSS
#define NO_RAID_BIT     0x0008          // Don't return any partitions that are on a RAID device

/* Pool list mask definitions */
#define NSS_POOL_BIT    0x0001          // Return all the pools on NSS partitions and RAIDs
#define LVM_POOL_BIT    0x0002          // Return all the pools on LVM partitions
#define SNAP_POOL_BIT   0x0004          // Return all the snapshot pools
#define SNAP_MOUNT_BIT  0x0008          // Return all the snapshot pools that are mounted
#define SHARED_POOL_BIT 0x0010          // Return pools that are shared
#define NSHARE_POOL_BIT 0x0020          // Return pools that are not shared
#define ALL_BITS        0xFFFF

/* Raidflags bit definitions */
#define ENABLED_BIT     0x01            // same as ENABLED flag
#define RESTRIPE_BIT    0x02            // Raid is restriping/remirroring
#define DEACTIVE_BIT    0x04            // Raid is deactive (failed raid)
#define SHARED_BIT      0x08            // Raid is shared for clustering
#define SUSPEND_BIT     0x10            // Raid is suspended    (dm suspend ioctl)
#define PAUSED_BIT      0x20            // Raid remirror is paused

/* Raid commands (used with NLVM_RaidCommand) */
#define RAID_ENABLE     1
#define RAID_DISABLE    2
#define RAID_RESTRIPE   3
#define RAID_ABORT      4
#define RAID_PAUSE      5
#define RAID_RESUME     6
#define RAID_FORCE      7

/* Pool info flag definitions */
#define SEGMENT_BIT     0x01            // Return the pool segment list
#define SNAPSHOT_BIT    0x02            // Return the pool snapshots

/* Pool state definitions */
#define POOL_DEACTIVE      2
#define POOL_MAINTENANCE   3
#define POOL_ACTIVE        6

/* NLVMClusterInfo_s flags */
#define NCP_ADVERTISE   0x01            // Enable NCP  (Defined from clstrlib.h DO NOT CHANGE)
#define CIFS_ADVERTISE  0x02            // Enable CIFS  (Defined from clstrlib.h DO NOT CHANGE)
#define AFP_ADVERTISE   0x04            // Enable AFP   (Defined from clstrlib.h DO NOT CHANGE)
#define UP_BIT          0x08            // The cluster is up
#define ACTIVATE_BIT    0x10            // Activate the pool
#define ENABLE_BIT      0x20

/* NLVMDeletedVolumeInfo_s states */
#define SALVAGEABLE     1
#define PURGE_PAUSED    2
#define PURGING         3

/* ChangeVolumeState states */
#define VOL_MOUNT       1
#define VOL_DISMOUNT    2
#define VOL_ACTIVATE    3
#define VOL_DEACTIVATE  4

/* Volume state definitions */
#define VOL_UNKNOWN     0
#define VOL_DEACTIVE    2
#define VOL_MAINTENANCE 3
#define VOL_ACTIVE      6
#define VOL_DELETED     16              // Only valid when calling NLVM_GetVolumeState

/* ChangeSnapshotState definitions */
#define SNAP_MOUNT      1
#define SNAP_DISMOUNT   2

/* Name space definitions */
#define DOS_NS          0
#define MAC_NS          1
#define UNIX_NS         2
#define LONG_NS         4

/* Volume supported namespace bits */
#define DOS_NS_BIT      (1<<DOS_NS)
#define MAC_NS_BIT      (1<<MAC_NS)
#define UNIX_NS_BIT     (1<<UNIX_NS)
#define LONG_NS_BIT     (1<<LONG_NS)

/* Volume attribute bits */
#define VOLATTR_SALVAGE                 0x00000001
#define VOLATTR_USER_SPACE_RESTRICTIONS 0x00000002
#define VOLATTR_READONLY                0x00000004
#define VOLATTR_COMPRESSION             0x00000008
#define VOLATTR_EXTENDED_ATTRIBUTES     0x00000010
#define VOLATTR_DATA_STREAMS            0x00000020
#define VOLATTR_DOS_METADATA            0x00000040
#define VOLATTR_NETWARE_METADATA        0x00000080
#define VOLATTR_MAC_METADATA            0x00000100
#define VOLATTR_UNIX_METADATA           0x00000200
#define VOLATTR_HARD_LINKS              0x00000400
#define VOLATTR_TRANSACTION             0x00000800
#define VOLATTR_SPARSE_FILES            0x00001000
#define VOLATTR_PHYSICAL_EOF            0x00002000
#define VOLATTR_DIRECT_IO               0x00004000
#define VOLATTR_PERSISTENT_ATTRIBUTES   0x00008000
#define VOLATTR_VERIFY                  0x00010000
#define VOLATTR_REBUILD                 0x00020000
#define VOLATTR_COW                     0x00040000      // file snapshot
#define VOLATTR_VIRTUAL_FILES           0x00080000
#define VOLATTR_USER_TRANSACTION        0x00100000
#define VOLATTR_USER_TRANSACTION_ACTIVE 0x00200000
#define VOLATTR_DONT_BACKUP             0x00400000
#define VOLATTR_MFL                     0x00800000
#define VOLATTR_DIR_QUOTAS              0x01000000
#define VOLATTR_SHRED_DATA              0x02000000
#define VOLATTR_SHARED                  0x04000000
#define VOLATTR_FLUSH_IMMEDIATE         0x08000000
#define VOLATTR_MIGRATION               0x10000000
#define VOLATTR_NCS_MASTER              0x20000000
#define VOLATTR_NCS_SLAVE               0x40000000
#define VOLATTR_ENCRYPTED               0x80000000
#define VOLATTR_EXTENDED_MAC_NAMESPACE  0x100000000ULL
#define VOLATTR_NO_ATIME                0x200000000ULL

#define VOL_NO_QUOTA                    0x7FFFFFFFFFFFF000ULL

/* Defines for NLVM_IsClusterEnabled type */
#define IS_POOL         0
#define IS_VOLUME       1

/* NLVM_Open flags */
#define OPEN_DEBUG      1

/* NLVM_Lock flags */
#define LOCK_CLEAR      0x01
#define LOCK_REFRESH    0x02
#define LOCK_NOSCAN     0x08            // WARNING: Use only for NLVM_RepairFromFile
#define LOCK_NOMOUNT    0x10            // Do not auto mount pools on discovery
// may also use SHARE_OVERRIDE

/* NLVM_InitializeDisk flags */ 
#define SET_SHARED      0x10000         // Set the device shared after initialization
#define SET_UNSHARED    0x20000         // Set the device as not shared after initialization

/* NLVM_Repair modes */
#define RB_NORM         1
#define RB_TEST         2
#define RB_FORCE        3

#define PARTITION_HEADER_SIZE   32
#define NAME_LENGTH             64
#define PATH_LENGTH             128
#define LABEL_LENGTH            128
#define GUID_SIZE               16
#define MAX_RAID_ELEMENTS       14
#define MAX_MIRROR_ELEMENTS     4
#define MAX_SIZE                -1ULL

typedef enum
{
        DV_ACTION_SALVAGE,
        DV_ACTION_PAUSE,
        DV_ACTION_CONTINUE,
        DV_ACTION_PURGE
} delVolAction_t;

typedef struct NLVMDeviceList_s
{
        char            name[NAME_LENGTH];
        uint64_t        size;
        uint64_t        freeSize;
        uint32_t        flags;
} NLVMDeviceList_s;

typedef struct NLVMDeviceInfo_s
{
        char            name[NAME_LENGTH];
        char            path[PATH_LENGTH];
        uint64_t        size;
        uint64_t        freeSize;
        uint64_t        usedSize;
        uint32_t        flags;
        uint32_t        major;
        uint32_t        minor;
        uint16_t        sectors;
        uint16_t        heads;
        uint8_t         devtype;
        uint8_t         shared;
        uint8_t         raidType;
        uint8_t         numSegments;
        uint8_t         loadedSegments;
        uint8_t         oldSegments;
        uint8_t         raidFlags;
        uint8_t         restripePercent;
        uint8_t         groupSync;
        uint32_t        partitionType;
        uint32_t        missingSegment;
        uint64_t        elementSize;
        uint32_t        stripeSize;
        uint32_t        membermap;
        uint32_t        inactivemap;
        uint32_t        syncmap;
        uuid_t          guid;
} NLVMDeviceInfo_s;

typedef struct NLVMPartitionList_s
{
        char            name[NAME_LENGTH];
        char            device[NAME_LENGTH];
        char            pool[NAME_LENGTH];
        uint64_t        start;
        uint64_t        size;
        uint32_t        type;
        uint32_t        flags;
        int             index;
        uint8_t         restripePercent;
} NLVMPartitionList_s;

typedef struct NLVMPartionInfo_s
{
        char            name[NAME_LENGTH];
        char            device[NAME_LENGTH];
        char            pool[NAME_LENGTH];
        char            raid[NAME_LENGTH];
        char            label[LABEL_LENGTH];
        uuid_t          guid;
        uint32_t        type;
        uint32_t        flags;
        uint64_t        start;
        uint64_t        size;                   // in sectors
        uint64_t        expandSize;             // in sectors
        uint32_t        major;
        uint32_t        minor;
        uint8_t         shared;
        uint8_t         restripePercent;
} NLVMPartitionInfo_s;

typedef struct NLVMSnapshotList_s
{
        char            name[NAME_LENGTH];
        char            pool[NAME_LENGTH];
        uint32_t        flags;
} NLVMSnapshotList_s;

typedef struct NLVMSnapshotInfo_s
{
        char            name[NAME_LENGTH];
        char            pool[NAME_LENGTH];
        char            partition[NAME_LENGTH];
        char            mountpoint[PATH_LENGTH];
        uint64_t        poolSize;
        uint64_t        partitionSize;
        uint32_t        chunkSize;
        uint32_t        percentFull;
        uint32_t        major;
        uint32_t        minor;
        uint32_t        flags;
} NLVMSnapshotInfo_s;

typedef struct NLVMPoolList_s
{
        char            name[NAME_LENGTH];
        uint64_t        size;
        uint32_t        flags;
} NLVMPoolList_s;

typedef struct NLVMPoolSegList_s
{
        char            name[NAME_LENGTH];
        char            device[NAME_LENGTH];
        uint32_t        index;
        uint64_t        offset;
        uint64_t        end;
} NLVMPoolSegList_s;

typedef struct NLVMPoolInfo_s
{
        char            name[NAME_LENGTH];
        char            mountpoint[PATH_LENGTH];
        char            originname[NAME_LENGTH];     // snapshot only
        uint64_t        size;
        int64_t         freeSize;
        uint64_t        usedSize;
        uint64_t        purgeSize;
        uint64_t        createTime;
        uint64_t        modTime;
        uint32_t        flags;
        uint32_t        state;
        uint32_t        blockSize;
        uuid_t          guid;
        uint32_t        major;
        uint32_t        minor;
        uint32_t        numVolumes;
        uint32_t        numSegments;
        uint32_t        percentFull;                // snapshot only
        uint32_t        listcount;
        NLVMPoolSegList_s *segments;
        uint32_t        snapcount;
        NLVMSnapshotList_s *snapshots;
} NLVMPoolInfo_s;

typedef struct NLVMVolumeList_s
{
        char            name[NAME_LENGTH];
} NLVMVolumeList_s;

/* The items listed as input (except name) can be changed by calling
** NLVM_ChangeVolumeInfo. All other fields are for info only.
*/
typedef struct NLVMVolumeInfo_s
{
        char            name[NAME_LENGTH];              //Input Output
        char            pool[NAME_LENGTH];              // Output
        char            mountpoint[PATH_LENGTH];        //Input Output
        int64_t         freeSize;                       // Output
        uint64_t        usedSize;                       // Output
        uint64_t        purgeSize;                      // Output
        uint64_t        quota;                          //Input Output
        time_t          createTime;                     // Output
        time_t          modTime;                        // Output
        time_t          archiveTime;                    // Output
        time_t          accessTime;                     // Output
        uint64_t        objects;                        // Output
        uint64_t        files;                          // Output
        uint64_t        compressedFiles;                // Output
        uint64_t        compressedDelFiles;             // Output
        uint64_t        uncompressFiles;                // Output
        uint64_t        precompressBytes;               // Output
        uint64_t        compressBytes;                  // Output
        uint64_t        supportedAttr;                  // Output
        uint64_t        enabledAttr;                    //Input Output
        uint32_t        state;                          //Input Output
        uint32_t        flags;                          //Input Output
        uint32_t        supportedNameSpace;             // Output
        uint32_t        nameSpace;                      //Input Output
        uint32_t        blockSize;                      // Output
        uint32_t        shredCount;                     //Input Output
        uint32_t        readAhead;                      //Input Output
        uint32_t        authModelID;                    // Output
        uint8_t         volumeID[16];                   // Output
        uint8_t         ndsID[16];                      // Output
} NLVMVolumeInfo_s;

typedef struct NLVMDeletedVolumeInfo_s
{
        char            name[NAME_LENGTH];
        char            origName[NAME_LENGTH];
        char            poolName[NAME_LENGTH];
        uint64_t        deleteTime;
        uint64_t        purgeTime;
        uint32_t        pursisState;
        int             state;
} NLVMDeletedVolumeInfo_s;

typedef struct NLVMVolumeCompression_s
{
        uint32_t        lastRatio;
        uint32_t        averageRatio;
        uint32_t        queuedReqs;
        uint32_t        queuedBGReqs;
        uint32_t        usedCompActivity;
        uint32_t        usedDecompActivity;
} NLVMVolumeCompression_s;

typedef struct NLVMLinuxVolumeInfo_s
{
        char            name[NAME_LENGTH];
        char            device[128];
        char            group[128];             // if lvm
        char            mountpoint[256];
        char            mkoptions[128];
        char            mntoptions[128];
        char            virtualServerName[128];
        char            type[16];
        uint64_t        size;
        uint32_t        flags;
        uint32_t        nameSpace;
        char            ipAddress[20];          // for clustering
} NLVMLinuxVolumeInfo_s;

/* Note:  The fromStatus and toStatus are defined as follows:
   Active (OK)
   WriteError
   SyncError
   ReadError
   UnknownError
   Installed (not functional)
   Missing  */
typedef struct NLVMMoveList_s
{
        char            name[NAME_LENGTH];
        char            pool[NAME_LENGTH];
        uint8_t         percent;
        char            fromStatus[16];
        char            toStatus[16];
} NLVMMoveList_s;

typedef struct NLVMMoveInfo_s
{
        char            name[NAME_LENGTH];
        char            pool[NAME_LENGTH];
        char            from[NAME_LENGTH];
        char            to[NAME_LENGTH];
        uint64_t        size;
        uint64_t        totalBlocks;
        uint64_t        doneBlocks;
        uint32_t        major;
        uint32_t        minor;
        uint8_t         percent;
        char            fromStatus[16];
        char            toStatus[16];
} NLVMMoveInfo_s;

typedef struct NLVMClusterInfo_s
{
        char            poolName[NAME_LENGTH];
        char            ndsName[256];
        char            virtualServerName[128];
        char            cifsVirtualServerName[20];
        char            ipAddress[20];
        char            fdn[514];
        uint32_t        flags;
} NLVMClusterInfo_s;

int NLVM_Open(int flags, char **errorptr);
int NLVM_Close(void);
int NLVM_Lock(int flags);
int NLVM_Unlock(void);
int NLVM_Refresh(int flags);
int NLVM_RepairFromFile(char *filename, int mode);

/* Device functions */
int NLVM_InitializeDisk(char *Device, int GPT, uint32_t flags);
int NLVM_SetDiskShared(char *Device, uint32_t flags);
int NLVM_GetDeviceList(NLVMDeviceList_s **list, int *count, int mask);
int NLVM_GetDeviceInfo(NLVMDeviceInfo_s *info);
/* Get a list of the partitions that are on this device */
int NLVM_GetDevicePartitionList(char *Device, NLVMPartitionList_s **list, int *count);
int NLVM_GetDevicePools(char *Device, NLVMPoolList_s **list, int *count);

/* Partition functions */
int NLVM_CreatePartition(NLVMPartitionInfo_s *partinfo);
int NLVM_ExpandPartition(char *Part, uint64_t size, uint32_t flags);
int NLVM_DeletePartition(char *Part, uint32_t flags);
int NLVM_ModifyLabel(char *Part, char *label, uint32_t flags);
int NLVM_GetPartitionList(NLVMPartitionList_s **list, int *count, int mask);
int NLVM_GetPartitionInfo(NLVMPartitionInfo_s *info);

/* Pool functions */
int NLVM_CreatePoolWithPartition(char *Pool, char *Part, NLVMClusterInfo_s *info, uint32_t flags);
int NLVM_CreatePool(char *Pool, NLVMDeviceList_s *list, int count, NLVMClusterInfo_s *info, uint32_t flags);
int NLVM_DeletePool(char *Pool, uint32_t flags);
int NLVM_ExpandPoolWithPartition(char *Pool, char *Part, uint32_t flags);
int NLVM_ExpandPool(char *Pool, NLVMDeviceList_s *list, int count, uint32_t flags);
int NLVM_RenamePool(char *Pool, char *NewName, char *NDSname, uint32_t flags);
int NLVM_MountPool(char *Pool, uint32_t flags);
int NLVM_UnmountPool(char *Pool);
int NLVM_GetPoolList(NLVMPoolList_s **list, int *count, uint32_t mask);
int NLVM_GetNextPoolName(unsigned long firstTime, int64_t *cookie, 
        char *poolName, unsigned long poolNameLen, unsigned long noSnapshot, 
        unsigned long *poolState, unsigned long *isShared, uint64_t *enabledAttr);
int NLVM_GetPoolInfo(NLVMPoolInfo_s *info, int segflag);
int NLVM_GetPoolState(char *poolName, unsigned long *state);
int NLVM_ChangePoolState(char *poolName, unsigned long newPoolState, uint32_t force);
int NLVM_IsPoolNameAcceptable(char *Pool);
int NLVM_GetPoolDeviceList(char *Pool, NLVMDeviceList_s **listptr, int *count);
int NLVM_GetVolumesOnPool(char *Pool, NLVMVolumeList_s **listptr, int *count, unsigned long deleted);
int NLVM_MovePool(char *Pool, NLVMDeviceList_s *list, int count, uint32_t flags);
int NLVM_DeleteMovePool(char *Pool, uint32_t flags);
int NLVM_GetMoveList(NLVMMoveList_s **list, int *count, uint32_t mask);
int NLVM_GetMoveInfo(NLVMMoveInfo_s *info);
int NLVM_GetPoolSnapshotFlags(char *Pool, uint32_t *flagptr);
int NLVM_GetGroupInfo(char *name, NLVMPoolSegList_s **groupinfo, int *count);

/* NSS Volume Functions */
int NLVM_CreateVolume(char *volumeName, char *poolName, char *password);
int NLVM_DeleteVolume(char *volumeName);
int NLVM_RenameVolume(char *volumeName, char *newName, char *password);
int NLVM_GetVolumeList(NLVMVolumeList_s **listptr, int *count);
int NLVM_GetVolumeInfo(NLVMVolumeInfo_s *volinfo);
int NLVM_GetVolumeState(char *volumeName, int *state);
int NLVM_GetDeletedVolumeInfo(NLVMDeletedVolumeInfo_s *info);
int NLVM_DeletedVolumeAction(char *Volume, delVolAction_t action);
int NLVM_GetVolumeHostPool(char *volume, char *pool);
int NLVM_ChangeVolumeInfo(NLVMVolumeInfo_s *volinfo);
int NLVM_ChangeVolumeState(char *volumeName, int newState, char *password);
int NLVM_GetCompressionStats(NLVMVolumeCompression_s *info);

/* Linux Volume Functions */
int NLVM_CreateLinuxVolume(NLVMLinuxVolumeInfo_s *volinfo);
int NLVM_DeleteLinuxVolume(char *name);
int NLVM_GetLinuxVolumeList(NLVMLinuxVolumeInfo_s **listptr, int *count);
int NLVM_GetLinuxVolumeFSTypes(char ***list, int *count);
int NLVM_ChangeLinuxMount(NLVMLinuxVolumeInfo_s *originfo, NLVMLinuxVolumeInfo_s *newinfo);

/* RAID functions */
int NLVM_CreateRaid(char *Raid, int raidType, int stripeSize, NLVMPartitionList_s *list, int count);
int NLVM_DeleteRaid(char *Raid);
int NLVM_ExpandRaid(char *Raid, NLVMPartitionList_s *list, int count);
int NLVM_RemoveRaidPartition(char *Raid, int index, uint32_t flags);
/* Get a list of the partitions that make up this RAID device */
int NLVM_GetRaidPartitionList(char *Raid, NLVMPartitionList_s **list, int *count);
int NLVM_RestripeRaid(char *Raid);
int NLVM_RenameRaid(char *Raid, char *newName);
int NLVM_RemoveMirror(char *Raid);
int NLVM_RaidCommand(char *Raid, int command);

/* NDS functions */
int NLVM_AddPoolToNDS(char *Pool, int shared, NLVMClusterInfo_s *info);
int NLVM_RemovePoolFromNDS(char *Pool, uint32_t flags);
int NLVM_GetPoolNDSName(char *Pool, char *ndsName);
int NLVM_RenamePoolNDSObject(char *Pool, char *newName);
int NLVM_GetServerNameFromNDS(char *serverName, char *container);
int NLVM_RemoveVolumeFromNDS(char *Volume, void *dfsGuid);
int NLVM_RemoveVolumeFromVLDB(void *dfsGuid);

/* Snapshot Functions */
int NLVM_CreateSnapshot(NLVMSnapshotInfo_s *snapinfo, char *Device, uint64_t size);
int NLVM_CreateSnapshotWithPartition(NLVMSnapshotInfo_s *snapinfo);
int NLVM_DeleteSnapshot(char *Snap);
int NLVM_GetSnapshotList(NLVMSnapshotList_s **listptr, int *count);
int NLVM_GetSnapshotInfo(NLVMSnapshotInfo_s *snapinfo);
int NLVM_ChangeSnapshotState(char *Snap, int newState, uint32_t flags);

/* Clustering Functions */
int NLVM_GetClusterName(char *clusterName, uint32_t *len);
int NLVM_GetClusterPoolNames(NLVMClusterInfo_s *info);
int NLVM_GetClusterPoolStatus(char *name, char *status, char *node);
int NLVM_IsClusterEnabled(char *name, int type, uint16_t *poolFDN, uint32_t len);
int NLVM_ClusterEnablePool(NLVMClusterInfo_s *info);
int NLVM_ClusterEnableVolume(char *fdn);
int NLVM_InitializeSBDPartition(char *Part);
int NLVM_SetClusterResource(char *resource, char *state, char *node);

/* Util routines */
char* NLVM_PartTypeToString(unsigned int type, char *description);
int NLVM_ValidateObjectName(char *name, int flags);

